
% Lepskii Method for Nonparametric Estimation of Effect of Class Size in Han (2019)

% DATE: 021319 


clear

load /[INSERT PATH HERE]/class_data.mat;

tic;  % Elapsed time


% --- Preliminaries

sz_penal = 50; % number of \tau_j

incr_penal = 0.1;

penal_0 = 0.1;  % tau_k; [0.001 0.005 0.01 0.05 0.1]
penal_j_0 = 0.1;
max_penal = incr_penal*sz_penal;
penal_vec = (penal_0:incr_penal:max_penal)';
penal_j_vec = penal_j_0:incr_penal:max_penal;
sz_penal_vec = size(penal_vec,1);


k_mat = [2 2; 3 3; 4 4; 5 5];
k3_mat = [4; 6; 8; 10];
graphcnt = 3;
k1 = k_mat(graphcnt,1);
k2 = k_mat(graphcnt,2);
k = k1 + k2;
k3 = k3_mat(graphcnt);
k_star = 0;

reps = 1000;

full = 0;   % Whether full sample (=1) full random subsample (=2) or discontinuity sample (=0)

% Normalization!
v_norm = 0; % normalizing value of v (-1, 1, 0, etc)

% Evaluation point of w can be changed!


% Variables:
% y : average verbal skill (average math skill)
% x : class size
% z : maimonides' rule (function of enrollment)
% w : percent disadvantaged (or, enrollment, enrollment2, linear trend)

n0 = size(a.avgverb,1);
n00 = sum(a.disc);
p = n00/n0;

if full == 1
    select = ones(n0,1);
elseif full == 2
    select = binornd(1,p,n0,1);
else
    select = a.disc;
end

n = sum(select);
y0 = zeros(n,1);
x0 = zeros(n,1);
z0 = zeros(n,1);
w0 = zeros(n,1);
schlcode = zeros(n,1);
cnt = 1;
for i = 1:n0
    if select(i)==1
    y0(cnt) = a.avgverb(i);
    x0(cnt) = a.classize(i);
    z0(cnt) = a.func1(i);
    w0(cnt) = a.tipuach(i);
    schlcode(cnt) = a.schlcode(i);
    cnt = cnt + 1;
    end
end

%y0 = y0 - mean(y0);
%x0 = x0 - mean(x0);


mu_x = mean(x0);
var_x = var(x0);

mu_z = mean(z0,1);  %row vector
var_z = var(z0);  %row vector

% Sample Size
n1 = size(y0,1);

y = y0;
x = x0;
z = z0;
w = w0;


    
n = size(y,1);

% --- Series Estimation using {yi,xi,vi_hat}

% Series Estimating RF
%{
Rmat = zeros(n,k3);
for i=1:k3;
    Rmat(:,i) = z.^(i-1); % from order ZERO
end
gamma = pinv(Rmat'*Rmat)*Rmat'*x;
v_hat2 = x - Rmat*gamma;
%}

% Linear RF
z1 = [ones(n,1) z w];
Pz = z1*pinv(z1'*z1)*z1';
Mz = eye(n) - Pz;
v_hat1 = Mz*x;
v_hat2 = v_hat1;


%%
% Estimating h(x) = g(x) + \lambda(v_hat)

% With Penalization

Pmat_w2 = zeros(n,1);  % CAUTION, dimension randomly assigned but it works!!
cnt = 1;
for i=0:k1-1
    for j=0:k1-1
        if i+j <= k1-1
        Pmat_w2(:,cnt) = x.^i .* w.^j; % from order ZERO
        cnt = cnt + 1;
        end
    end
end
k_b1 = cnt - 1;
for j=1:k2-1 % CAUTION: NOT j=k1+1:k, also drop intercept term (so power is j below)
    Pmat_w2(:,cnt) = v_hat2.^j; % from order ZERO as well % CAUTION, column
    cnt = cnt + 1;
end
k_b = cnt - 1;
Pmat = Pmat_w2;

% Start loop for diff penality
mat_bias_in_T = zeros(sz_penal_vec,sz_penal_vec);
mat_var_in_T = zeros(sz_penal_vec,sz_penal_vec);
mat_diff_in_T = zeros(sz_penal_vec,sz_penal_vec);
for pp = 1:sz_penal_vec
    penal = penal_vec(pp);  % penal = tau_k, change value to search

    D_star = eye(k_b);
    if k_star~=0
        for i=1:(k_star+1)/2   % Caution: control intercept and first (k_star-1)/2 coefficent
            D_star(i,i) = 0;
        end
        for i=k1+1:k1+(k_star-1)/2   % Caution: control first (k_star-1)/2 coefficent starting from k1+1; When k_star=1, interval is not defined as desired
            D_star(i,i) = 0;
        end
    end
    beta2 = pinv(Pmat'*Pmat + n*penal*D_star)*Pmat'*y;

    % --- Calculating data-driven \tau (100719)

    PP_tau = Pmat'*Pmat + n*penal*D_star;   % penal = tau_k, change value to search

    var_py_0 = zeros(n,k_b);
    mu_py = Pmat'*y/n;
    %mu_py = mean(mu_py0)';
    for i=1:n
        var_py_0(i,:) = (Pmat(i,:)'*y(i)-mu_py)';
    end
    var_py = var_py_0'*var_py_0/n;

    for qq = 1:sz_penal_vec
        penal_j = penal_j_vec(qq);
        beta2_alt = pinv(Pmat'*Pmat + n*penal_j*D_star)*Pmat'*y;  % with increment in penalty (penal1+0.01 = tau_j)
        penal_bias_feas = beta2_alt - beta2;

        if penal <= penal_j
            bias_in_T = norm(penal_bias_feas, 'fro');
            var_first = norm( (eye(k_b)-pinv(Pmat'*Pmat/n + penal*D_star)*(Pmat'*Pmat/n))*beta2, 'fro');
            var_second = norm( (eye(k_b)-pinv(Pmat'*Pmat/n + penal_j*D_star)*(Pmat'*Pmat/n))*beta2_alt, 'fro');
            var_in_T = 2*(var_first+var_second);
            diff_in_T = var_in_T - bias_in_T;
        else
            bias_in_T = 111111;
            var_in_T = 111111;
            diff_in_T = 111111;
        end
        mat_bias_in_T(pp,qq) = bias_in_T;
        mat_var_in_T(pp,qq) = var_in_T;
        mat_diff_in_T(pp,qq) = diff_in_T;

    % END LOOP for penal_j (qq)
    end

% END LOOP for penal (pp)
end

RESULT = [[111111 penal_j_vec]; [penal_vec mat_diff_in_T]];



tElapsed=toc/60
