
% Monte Carlo Simulations for Data-Driven Choice of Penalization Parameter in Han (2019)

% DATE: 021319 

% y = g(x) + e
% x = z*\pi + v
% Draw z from Normal
% Draw (e,v)' from BVN

clear

tic;  % Elapsed time

s = 1;   % The number of simulation estimating g0
n = 2000;   % The sample size in each simulation

rng(5); % seed


% --- Preliminaries


rho = 0.5; % Degree of Endog

%lambda = [2 4 8 16 32 64 128 256];  % STRENGTH OF IV
lambda = 64;

sz_penal = 200; % number of \tau_j

incr_penal = 0.0001;    % 0.0001 BEST 021019 

penal_0 = 0.0001;  % tau_k; [0.001 0.005 0.01 0.05 0.1]
penal_j_0 = 0.0001;
max_penal = incr_penal*sz_penal;
penal_vec = (penal_0:incr_penal:max_penal)';
penal_j_vec = penal_j_0:incr_penal:max_penal;
sz_penal_vec = size(penal_vec,1);

k1 = 6; % Order of approx functions for g
k2 = 6; % Order of approx functions for \lambda
k = k1 + k2;
k_star = 1; % k_star should be 0 or 1,3,5,7,... (i.e., of the form 2n-1), and smaller than k-1


% -----

mu_x = 2;
var_x = 3;  % This should be big enough, o.w. var_v becomes negative

mu_z = 0;
var_z = 1;

% Calculating pi
pi_all = sqrt(lambda./((var_z + mu_z^2)*n));


%% Calculating true values

mu_py0 = zeros(s,k-1);
Q0 = zeros(s,k-1,k-1);
%{
for s = 1:1000

    pi = pi_all(1);  % pi from lambda

    mu = [0 0];
    var_e = 1;
    var_v = 1;  % CAUTION
    cov = rho*sqrt(var_e)*sqrt(var_v);
    sigma = [var_e cov;cov var_v];    
    ev = mvnrnd(mu,sigma,n); % RNG from bivariate normal
    v_eval = ev(:,2);

    
    mu_cdf = mu_x;
    sd_cdf = sqrt(var_x);

    % GENERATE z, pi, e, v for x
    z = mu_z + sqrt(var_z)*randn(n,1); % RND from normal

    mu = [0 0];
    var_e = 1;
    var_v = 1;  % CAUTION
    cov = rho*sqrt(var_e)*sqrt(var_v);
    sigma = [var_e cov;cov var_v];    
    ev = mvnrnd(mu,sigma,n); % RNG from bivariate normal
    e = ev(:,1);
    v = ev(:,2);

    % GENERATE x:  Invariant to simulation repetition
    x = mu_x - mu_z*pi + z*pi + v; % x has same mean indep of pi

    % yi : Generate via y = g(x) + e

    % g0(x) = cdfn((x-mu)/var)
    g0 = cdf('normal',x,mu_cdf,sd_cdf); % CAUTION: sd, NOT variance
    y = g0 + e;


    % --- Series Estimation using {yi,xi,vi_hat}

    % Estimating \pi and vi
    % CAUTION:  INTERCEPT
    z1 = [ones(n,1) z];
    pi_hat = pinv(z1'*z1)*z1'*x; % a\b instead of inv(a)*b (also consider pinv)
    Pz = z1*pinv(z1'*z1)*z1';
    Mz = eye(n) - Pz;
    v_hat = Mz*x;
    v_hat_check = x - z1*pi_hat;


    % Estimating h(x) = g(x) + \lambda(v_hat)

    % Series Estimation
    % w = (x,v)
    % At this stage, only one intercept in Pmat_w
    Pmat = zeros(n,k-1); % k-1, only one intercept
    for i=1:k1
        Pmat(:,i) = x.^(i-1); % from order ZERO
    end
    for j=1:k2-1 % CAUTION: NOT j=k1+1:k, also drop intercept term (so power is j below)
        Pmat(:,k1+j) = v_hat.^j; % from order ZERO as well % CAUTION, column
    end

    mu_py0(s,:) = (Pmat'*y/n)';
    Q0(s,:,:) = Pmat'*Pmat/n;
    %mu_py = mean(mu_py0)';

end

mu_py0 = mean(mu_py0,1);
mu_py0 = mu_py0';
Q0 = squeeze(mean(Q0,1));
%}


%% START LOOP for variation in pi

sz_l = size(lambda,2);
vec_g1 = zeros(s,sz_l*n);  % Collect ghat_cf
vec_g2 = zeros(s,sz_l*n);  % Collect ghat_cf
vec_g_ls = zeros(s,sz_l*n);  % Collect ghat_ls
vec_eig = zeros(k-1,sz_l);    % Collect eigen value of Q hat
vec_penal_b_v = zeros(4,sz_l);  % Collect bias and var from penalization

for kk = 1:sz_l
    pi = pi_all(kk);  % pi from lambda

    mu = [0 0];
    var_e = 1;
    var_v = 1;  % CAUTION
    cov = rho*sqrt(var_e)*sqrt(var_v);
    sigma = [var_e cov;cov var_v];    
    ev = mvnrnd(mu,sigma,n); % RNG from bivariate normal
    v_eval = ev(:,2);

    
    mu_cdf = mu_x;
    sd_cdf = sqrt(var_x);

    % GENERATE z, pi, e, v for x
    z = mu_z + sqrt(var_z)*randn(n,1); % RND from normal

    mu = [0 0];
    var_e = 1;
    var_v = 1;  % CAUTION
    cov = rho*sqrt(var_e)*sqrt(var_v);
    sigma = [var_e cov;cov var_v];    
    ev = mvnrnd(mu,sigma,n); % RNG from bivariate normal
    e = ev(:,1);
    v = ev(:,2);

    % GENERATE x:  Invariant to simulation repetition
    x = mu_x - mu_z*pi + z*pi + v; % x has same mean indep of pi

    % yi : Generate via y = g(x) + e

    % g0(x) = cdfn((x-mu)/var)
    g0 = cdf('normal',x,mu_cdf,sd_cdf); % CAUTION: sd, NOT variance
    y = g0 + e;


    % --- Series Estimation using {yi,xi,vi_hat}

    % Estimating \pi and vi
    % CAUTION:  INTERCEPT
    z1 = [ones(n,1) z];
    pi_hat = pinv(z1'*z1)*z1'*x; % a\b instead of inv(a)*b (also consider pinv)
    Pz = z1*pinv(z1'*z1)*z1';
    Mz = eye(n) - Pz;
    v_hat = Mz*x;
    v_hat_check = x - z1*pi_hat;


    % Estimating h(x) = g(x) + \lambda(v_hat)

    % Series Estimation
    % w = (x,v)
    % At this stage, only one intercept in Pmat_w
    Pmat = zeros(n,k-1); % k-1, only one intercept
    for i=1:k1
        Pmat(:,i) = x.^(i-1); % from order ZERO
    end
    for j=1:k2-1 % CAUTION: NOT j=k1+1:k, also drop intercept term (so power is j below)
        Pmat(:,k1+j) = v_hat.^j; % from order ZERO as well % CAUTION, column
    end

    % Estimation
    % Let \beta be the corresponding parameter
    beta1 = pinv(Pmat'*Pmat)*Pmat'*y;
    
    % PENALIZED Series Estimation

    % Start loop for diff penality
    mat_bias_in_T = zeros(sz_penal_vec,sz_penal_vec);
    mat_var_in_T = zeros(sz_penal_vec,sz_penal_vec);
    mat_diff_in_T = zeros(sz_penal_vec,sz_penal_vec);
    for pp = 1:sz_penal_vec
        penal = penal_vec(pp);  % penal = tau_k, change value to search

        D_star = eye(k-1);
        if k_star~=0
            for i=1:(k_star+1)/2   % Caution: control intercept and first (k_star-1)/2 coefficent
                D_star(i,i) = 0;
            end
            for i=k1+1:k1+(k_star-1)/2   % Caution: control first (k_star-1)/2 coefficent starting from k1+1; When k_star=1, interval is not defined as desired
                D_star(i,i) = 0;
            end
        end
        beta2 = pinv(Pmat'*Pmat + n*penal*D_star)*Pmat'*y;

        % --- Calculating eigenvalues of \hat{Q} (100719)

        vec_eig(:,kk) = eig(Pmat'*Pmat/n);

        % --- Calculating data-driven \tau (100719)
        %D_star = eye(k-1);
        PP_tau = Pmat'*Pmat + n*penal*D_star;   % penal = tau_k, change value to search
        Q_tau0 = Q0 + penal*D_star;
        Q_hat_tau = Pmat'*Pmat/n + penal*D_star;

        var_py_0 = zeros(n,k-1);
        mu_py = Pmat'*y/n;
        
        for i=1:n
            var_py_0(i,:) = (Pmat(i,:)'*y(i)-mu_py)';
        end
        var_py = var_py_0'*var_py_0/n;

        for qq = 1:sz_penal_vec
            penal_j = penal_j_vec(qq);
            beta2_alt = pinv(Pmat'*Pmat + n*penal_j*D_star)*Pmat'*y;  % with increment in penalty (penal1+0.01 = tau_j)
            penal_bias_feas = beta2_alt - beta2;
            penal_var7 = sqrt( trace(pinv(PP_tau/n)*pinv(PP_tau/n)*var_py) );
            
            PP_tau_j = Pmat'*Pmat + n*penal_j*D_star;
            Q_tau_j0 = Q0 + penal_j*D_star;
            
            eig_max = max(eig(pinv(Pmat'*Pmat/n + penal*D_star)));

            if penal <= penal_j
                bias_in_T = norm(penal_bias_feas, 'fro');
                var_first = norm( (eye(k-1)-pinv(Q_hat_tau)*(Pmat'*Pmat/n))*beta2, 'fro');
                var_second = norm( (eye(k-1)-pinv(Pmat'*Pmat/n + penal_j*D_star)*(Pmat'*Pmat/n))*beta2_alt, 'fro');
                var_in_T = 2*(var_first+var_second);
                diff_in_T = var_in_T - bias_in_T;   % search tau for this diff to be positive
            else
                bias_in_T = 11111;
                var_in_T = 11111;
                diff_in_T = 11111;
            end
            mat_bias_in_T(pp,qq) = bias_in_T;
            mat_var_in_T(pp,qq) = var_in_T;
            mat_diff_in_T(pp,qq) = diff_in_T;

        % END LOOP for penal_j (qq)
        end

    % END LOOP for penal (pp)
    end   
    
% END LOOP for pi (kk)
end
    

RESULT = [[11111 penal_j_vec]; [penal_vec mat_diff_in_T]];

clear Mz Pk Pk1 Pk2 Pmat_w Pz % Pmat

tElapsed=toc/60;